/*
        Disclaimer: The code is provided as it is, with no warranties.
        Mag Hoehme <java@mhoehme.de>, 2014

*/
package de.mhoehme.android.partyfinder;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.provider.Settings;
import android.util.Log;
import android.widget.TextView;

import java.util.Date;

/**
 * Simple activity to get the current location and display it.
 *
 * It demonstrates how the last known location can be used to improve the responsiveness of the app.
 */
public class LocationActivity extends Activity implements LocationListener {

    private static final String TAG = LocationActivity.class.getName();


    private LocationManager  locationManager;
    private String           provider;


    /**
     * Called when the activity is first created.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.location_layout);

        // (1) Get the location manager
        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);


        // enableGpsIfNotEnabled();


        // (2) Choose GPS as location provider
        provider = LocationManager.GPS_PROVIDER;

        //     or get the best enabled provider:
        //     - criteria must not be null
        //     - true -> only those that are enabled
        // provider = locationManager.getBestProvider(new Criteria(), true);


        // (3) Get the last known location to have an immediate starting point
        Location location = locationManager.getLastKnownLocation(provider);

        Log.d(TAG, "[onCreate] last known location = " + location);

        process(location);
    }



    /**
     * Request updates at startup
     */
    @Override
    protected void onResume() {
        super.onResume();

        // (4) Request the location manager to fire updates every 5 seconds
        //     or if the location has changed more than 1 m
        locationManager.requestLocationUpdates(provider, 5000, 1, this);
    }


    /**
     * Remove the locationlistener updates when LocationActivity is paused to save battery life
     */
    @Override
    protected void onPause() {
        super.onPause();
        locationManager.removeUpdates(this);
    }


    // -- LocationListener ----------------------------------------------------------------------

    @Override
    public void onLocationChanged(Location location) {
        Log.d(TAG, "[onLocationChanged] location = "+location);

        // (5) Process location
        process(location);
    }


    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) { }

    @Override
    public void onProviderEnabled(String provider) { }

    @Override
    public void onProviderDisabled(String provider) { }


    // --  ----------------------------------------------------------------------

    /**
     * Altitude and speed depend on the device.
     */
    private void process(Location location) {

        if (location == null) {

            display(R.id.time,      new Date().toLocaleString());
            display(R.id.provider,  "Provider: ");
            display(R.id.latitude,  "Latitude: ");
            display(R.id.longitude, "Longitude: ");

            display(R.id.altitude,  "Altitude: ");
            display(R.id.speed,     "Speed: ");

            display(R.id.accuracy,  "Accuracy: ");

        } else {

            display(R.id.time,                        new Date(location.getTime()).toLocaleString());
            display(R.id.provider,  "Provider: "    + location.getProvider());
            display(R.id.latitude,  "Latitude: "    + Location.convert(location.getLatitude(), Location.FORMAT_DEGREES));
            display(R.id.longitude, "Longitude: "   + Location.convert(location.getLongitude(), Location.FORMAT_DEGREES));

            display(R.id.altitude,  "Altitude: "    + String.valueOf(location.getAltitude()) + " m");
            display(R.id.speed,     "Speed: "       + location.getSpeed() + "m/s");

            display(R.id.accuracy,  "Accuracy: +/-" + String.valueOf(location.getAccuracy()) + " m");

        }
    }


    private void display(int id, String text) {
        TextView tv = (TextView)findViewById(id);
        tv.setText(text);
    }



    // -- Request user to enable GPS -----------------------------------------------------------

    /**
     * Checks if GPS provider is enabled or not.
     * If the GPS provider is not enabled, an intent will be fired.
     */
    private void enableGpsIfNotEnabled() {
        if (!locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
            // Fire an indirect intent that pops up the GPS settings dialog

            // Settings.ACTION_LOCATION_SOURCE_SETTINGS:
            // Activity Action: Show settings to allow configuration of current location sources.
            // In some cases, a matching Activity may not exist, so ensure you safeguard against this.

            Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
            startActivity(intent);
        }
    }
}
