/*
        Disclaimer: The code is provided as it is, with no warranties.
        Mag Hoehme <java@mhoehme.de>, 2014

*/
package de.mhoehme.android.partyfinder;

import android.app.Activity;

import android.content.Context;
import android.location.Criteria;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import de.mhoehme.android.partyfinder.model.DataProvider;
import de.mhoehme.android.partyfinder.model.Place;
import de.mhoehme.android.partyfinder.overlays.MyLocationOverlay;
import org.osmdroid.DefaultResourceProxyImpl;
import org.osmdroid.ResourceProxy;
import org.osmdroid.tileprovider.tilesource.TileSourceFactory;
import org.osmdroid.util.GeoPoint;
import org.osmdroid.views.MapView;
import org.osmdroid.views.overlay.*;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * (1) Get osmdroid from download page
 *     https://code.google.com/p/osmdroid/wiki/Downloads
 *
 * (2) Include slf4j
 *     http://www.slf4j.org/download.html
 *
 *
 *
 * http://www.gleisarbeiter.de/2011/03/17/using-openstreetmaps-with-osmdroid-on-android/
 *
 * http://android-coding.blogspot.de/2012/06/example-of-implementing-openstreetmap.html
 */
public class PartyMapActivity extends Activity implements LocationListener {

    private static final String TAG = PartyMapActivity.class.getName();


    private static final int ZOOM = 14;


    private MyLocationOverlay   myLocationOverlay   = null;
    private ItemizedIconOverlay itemizedIconOverlay = null;



    /**
     * Called when the activity is first created.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.map_layout);


        final MapView mapView = (MapView) findViewById(R.id.openmapview);
        mapView.setClickable          (true);
        mapView.setBuiltInZoomControls(true);
        mapView.setTileSource         (TileSourceFactory.MAPNIK);

        mapView.getController().setZoom(ZOOM);                         // http://wiki.openstreetmap.org/wiki/Zoom_levels



        // -- Itemized Overlay  -----------------------------------------

        itemizedIconOverlay = getPartiesOverlay();

        mapView.getOverlays().add(itemizedIconOverlay);



        // -- MyLocationOverlay -----------------------------------------

        // http://android-coding.blogspot.de/2012/06/example-of-implementing-openstreetmap.html
        // http://android-coding.blogspot.de/2012/07/display-user-location-by-adding.html
        // http://android-coding.blogspot.de/2012/07/osmdroid-mapview-to-follow-user.html
        myLocationOverlay = new MyLocationOverlay(this, mapView);

        myLocationOverlay.addListener(this);


        // https://developers.google.com/maps/documentation/android/v1/reference/com/google/android/maps/MyLocationOverlay#runOnFirstFix%28java.lang.Runnable%29
        //
        // Queues a runnable to be executed as soon as we have a location fix.
        // If we already have a fix, we'll execute the runnable immediately and
        // return true. If not, we'll hang on to the runnable and return false;
        // as soon as we get a location fix, we'll run it in in a new thread.
        //
        // mho: runs as soon as the position changes
        myLocationOverlay.runOnFirstFix(new Runnable() {
            public void run() {

                GeoPoint myLocation = myLocationOverlay.getMyLocation();

                if (myLocation != null) {
                    Log.d(TAG, "[runOnFirstFix.run] location = "+myLocation.getLatitude()+" "+myLocation.getLongitude()
                            +" ("+myLocation+")");
                    mapView.getController().animateTo(myLocation);
                }

            }
        });

        mapView.getOverlays().add(myLocationOverlay);




        // -- ScaleBarOverlay -----------------------------------------

        // (1) Create the overlay
        ScaleBarOverlay myScaleBarOverlay = new ScaleBarOverlay(this);

        // (2) add it to the mapView
        mapView.getOverlays().add(myScaleBarOverlay);



        // -- MinimapOverlay  -----------------------------------------

        // (1) Create and initialize the overlay
        MinimapOverlay miniMapOverlay = new MinimapOverlay(this, mapView.getTileRequestCompleteHandler());
        miniMapOverlay.setZoomDifference(  4);
        miniMapOverlay.setHeight        (100);
        miniMapOverlay.setWidth         (100);

        // (2) add it to the mapView
        mapView.getOverlays().add(miniMapOverlay);


        moveToLastKnownLocation();
    }


    @Override
    protected void onResume() {

        super.onResume();

        myLocationOverlay.enableMyLocation();
        myLocationOverlay.enableFollowLocation();

    }


    /**
     * Switch off the overlays we do not need to save power
     */
    @Override
    protected void onPause() {

        super.onPause();

        myLocationOverlay.disableMyLocation();
        myLocationOverlay.disableFollowLocation();
    }


    /**
     * Get list of parties within the given distance and return them
     * as OverlayItems
     * @param location current location
     * @param distance distance
     * @return list of OverlayItems
     */
    private List<OverlayItem> getParties(Location location, int distance) {
        final List<OverlayItem> parties = new ArrayList<>();
        for (Place place : DataProvider.getParties(location, distance)) {
            parties.add(new OverlayItem(
                            place.getTitle(),
                            place.getInfo(),
                            new GeoPoint(place.getLatitude(), place.getLongitude())));
        }

        return parties;
    }



    /**
     * Creates an itemized overlay with an empty list.
     * The list will be populated depending on the user's location.
     *
     * @return an overlay with the icons
     */
    private ItemizedIconOverlay getPartiesOverlay() {

        Drawable marker  = getResources().getDrawable(R.drawable.bluefrog_small);
        int markerWidth  = marker.getIntrinsicWidth();
        int markerHeight = marker.getIntrinsicHeight();
        marker.setBounds(0, markerHeight, markerWidth, 0);


        ItemizedIconOverlay.OnItemGestureListener<OverlayItem> listener = new ItemizedIconOverlay.OnItemGestureListener<OverlayItem>() {

            /**
             * Displays a toast with information of the clicked party
             * @param index the index of the clicked item
             * @param item  the overlay item that was clicked
             * @return always true
             */
            @Override
            public boolean onItemSingleTapUp(int index, OverlayItem item) {
                String distance = "";
                if (myLocationOverlay.getMyLocation() != null) {

                    // calculate the distance to the selected party as m or km
                    int d = item.getPoint().distanceTo(myLocationOverlay.getMyLocation());
                    if (d < 1000) {
                        distance = d + " m";
                    } else {
                        BigDecimal bd = new BigDecimal(Float.toString(d));
                        bd = bd.divide(new BigDecimal("1000"));
                        bd = bd.setScale(2, BigDecimal.ROUND_HALF_UP);
                        distance = bd.floatValue() + " km";
                    }
                }

                String text = item.getTitle() + "\n"
                        + item.getSnippet() + "\n"
                        + Location.convert(item.getPoint().getLatitude(),  Location.FORMAT_DEGREES) + " "
                        + Location.convert(item.getPoint().getLongitude(), Location.FORMAT_DEGREES) + "\n"
                        + distance;

                // Toast.makeText(PartyMapActivity.this, text, Toast.LENGTH_LONG).show();

                // -- Using a custom toast ---------------------------------------------------
                //    http://developer.android.com/guide/topics/ui/notifiers/toasts.html

                LayoutInflater inflater = getLayoutInflater();
                View layout = inflater.inflate(R.layout.custom_toast_layout,
                        (ViewGroup) findViewById(R.id.toast_layout_root));

                TextView tv = (TextView) layout.findViewById(R.id.text);
                tv.setText(text);

                Toast toast = new Toast(getApplicationContext());
                toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
                toast.setDuration(Toast.LENGTH_LONG);
                toast.setView(layout);
                toast.show();

                return true;
            }

            @Override
            public boolean onItemLongPress(int index, OverlayItem item) {
                return false;
            }
        };


        ResourceProxy resourceProxy = new DefaultResourceProxyImpl(getApplicationContext());

        return new ItemizedIconOverlay(new ArrayList<OverlayItem>(), marker, listener, resourceProxy);

        // Using the default marker: big green bubbles with hand signs
        // return new ItemizedOverlayWithFocus(new ArrayList<OverlayItem>(), listener, resourceProxy);
    }





    /**
     * Move to the last known location to improve the responsiveness of your app
     */
    private void moveToLastKnownLocation() {

        LocationManager locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);

        String provider = locationManager.getBestProvider(new Criteria(), true);

        Location lastKnownLocation =  locationManager.getLastKnownLocation(provider);

        Log.d(TAG, "[moveToLastKnownLocation] location = "+lastKnownLocation);

        if (lastKnownLocation != null) {
            // Centering the map to the last known location...
            //    mapView.getController().setCenter(new GeoPoint(lastKnownLocation.getLatitude(), lastKnownLocation.getLongitude()));
            // or settling the overlay to the last known location (map will center automatically):
            myLocationOverlay.setLocation(lastKnownLocation);

            onLocationChanged(lastKnownLocation);
        }
    }



    // -- Implementation of LocationListener ---------------------------------------------

    @Override
    public void onLocationChanged(Location location) {

        // TODO this could be improved...
        itemizedIconOverlay.removeAllItems();

        List<OverlayItem> parties = getParties(location, 50 * 1000);
        if (parties.size() > 0) {
            itemizedIconOverlay.addItems(parties);
        }

        ((MapView) findViewById(R.id.openmapview)).getController().
                animateTo(new GeoPoint(location.getLatitude(), location.getLongitude()));
    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {

    }

    @Override
    public void onProviderEnabled(String provider) {

    }

    @Override
    public void onProviderDisabled(String provider) {

    }
}
